//////////////////////////////////////////////////////////////////////////////
//                      Copyright (c) 2005 Gerhard Schalk
//
//                      80C537A Mikrocontroller Training
//                              C Programmierung
//
//////////////////////////////////////////////////////////////////////////////
//   Projekt:       Dallas DS1822 1-Wire Digital Thermometer
//                  Beim 1-Wire Bus ist der uC immer der Master und
//                  die DS1822 Sensoren sind die Slaves.
//
//                  Dieses Beispiel setzt ein Single-drop-System (also nur 
//                  einen DS1822) und eine Spannungsversorgung am Vdd Pin
//                  des DS1822 voraus.
//
//
//   Filename:      DS1822.C
//////////////////////////////////////////////////////////////////////////////   

//-------------------------------------------------------------------------
//  Include Dateien
//-------------------------------------------------------------------------
#include <stdio.h>          // Standard I/O Funktionen
#include "REG517A.H"
#include "DS1822.H"



//--------------------------------------------------------------------------
// Funktion: char DS1822ReadROMCode(unsigned char *RomCodeData)
//
// Diese Funktion liest das 64-Bit laserd DS1822 ROM aus.
// Im Rom sind folgende Daten enthalten:
// MSB |8-Bit CRC | 48-Bit Serial Number |8-Bit Family Code (0x22) | LSB
//--------------------------------------------------------------------------
#if 0
char DS1822ReadROMCode(unsigned char *RomCodeData)
{
    unsigned char i;
    char Status;


    Status =  OneWire_Reset();
    if(Status == DS1822_NotPresent)
        return(ERROR);

    // Schreiben des ReadROM Commando Byte
    OneWireWriteByte(ReadROMCommando);
    
    // 8-Byte Rom Code Daten lesen
    // (8-Bit Family Code)
    // (48-Bit Serial Number)
    // (8-Bit CRC)
    for (i=0; i<8; i++)
    {
        RomCodeData[i] = OneWireReadByte();
    }

    // CRC Ueberpruefung noch nicht implementiert
    return OK;
    
}
#endif

//--------------------------------------------------------------------------
// Funktion: char DS1822ReadTemp(unsigned char *Temp)
//
// Diese Funktion startet eine Temp. Wandlung 
// und liest die aktuelle Temperatur aus.
//--------------------------------------------------------------------------
char DS1822ReadTemp(float *TempValue)
{
    unsigned char i;
    char Status;
    unsigned char ScratchpadRAM[9];
    unsigned int tempInt;

    Status =  OneWire_Reset();          // Reset
    if(Status == DS1822_NotPresent)
        return(ERROR);
    
    OneWireWriteByte(SkipROM);          // Write Cmd
    OneWireWriteByte(ConvertTemp);      // Write Cmd

    OneWireDelay(15);
    
    Status =  OneWire_Reset();          // Reset
    if(Status == DS1822_NotPresent)
        return(ERROR);   
    OneWireWriteByte(SkipROM);          // Write Cmd
    OneWireWriteByte(ReadScratchpad);   // Write Cmd   

    // 8 Byte Scratchpad Daten lesen 
    for (i=0; i<9; i++)
    {
        ScratchpadRAM[i] = OneWireReadByte();
    }

    //-------------------------------------------------
    // Dallas 16-Bit Format:
    // Temp. MSB:  S   S   S   S   S   2^6  2^5  2^4
    // Temp. LSB: 2^3 2^2 2^1 2^0 2^-1 2^-2 2^-3 2^-4
    //-------------------------------------------------

    // In 32-Bit Float Zahl wandeln 
    if(ScratchpadRAM[1] & 0x80) // Vorzeichenbit S testen 
    {
        // Neg. Temperaturwert
        tempInt = (ScratchpadRAM[1] << 8) + ScratchpadRAM[0]; 
        tempInt = ~tempInt + 1;  // 16-Bit Zweierkomplement bilden
        *TempValue = -1.0f * ((float)(tempInt >> 4) + (tempInt & 0x000F) * 0.0625f);
    }
    else
    {   
        // Pos. Temperaturwert
        *TempValue = (float)((ScratchpadRAM[1] & 0x07) << 4) + 
                     (float)((ScratchpadRAM[0]&0xF0) >> 4) +
                     ((ScratchpadRAM[0] & 0x0F) * 0.0625f);
    }    
    return OK;
}


//--------------------------------------------------------------------------
// Funktion: char DS1822ReadScratchpadRAM(void)
//
// Diese Testfunktion liest alle 9 Byte des Scratchpad RAM aus und gibt
// alle Werte via RS232 aus.
//--------------------------------------------------------------------------
#if 0
char DS1822ReadScratchpadRAM(void)
{
    unsigned char i;
    char Status;
    unsigned char ScratchpadRAM[9];

    Status =  OneWire_Reset();          // Reset
    if(Status == DS1822_NotPresent)
        return(ERROR);
    
    OneWireWriteByte(SkipROM);          // Write Cmd
    OneWireWriteByte(ConvertTemp);      // Write Cmd

    OneWireDelay(15);
    
    Status =  OneWire_Reset();          // Reset
    if(Status == DS1822_NotPresent)
        return(ERROR);   
    OneWireWriteByte(SkipROM);          // Write Cmd
    OneWireWriteByte(ReadScratchpad);   // Write Cmd   

    // 8 Byte Scratchpad Daten lesen 
    for (i=0; i<9; i++)
    {
        ScratchpadRAM[i] = OneWireReadByte();
    }
    for(i=0; i<9; i++)
        printf("Scratchpad RAM Byte: %b02X Value: %b02X Hex\n",i,ScratchpadRAM[i]);
    return OK;
}
#endif

//--------------------------------------------------------------------------
// Funktion: unsigned char OneWire_Reset(void)
//
// Diese Funktion erzeugt einen Reset-Impuls und gibt den Status des
// DS1822 Slave PresenceImpulses zurueck. 
// 0 ..  mind. ein 1-Wire Slave ist am Bus angeschlossen 
// 1 ..  kein 1-Wire Slave ist am Bus angeschlossen 
//--------------------------------------------------------------------------
unsigned char OneWire_Reset(void)
{
    unsigned char SlavePresent;
    
    // Erzeugen des Resetimpulses (ca. 480us)
    DQ = LOW;           // pull DQ line low
    OneWireDelay(29);   // leave it low for 480us
    DQ = HIGH;          // allow line to return high

    // Pruefen auf Slave Presence Signal     
    OneWireDelay(3);       // wait for presence
    SlavePresent = DQ;     // get presence signal
    OneWireDelay(25);      // wait for end of timeslot
    return(SlavePresent);  // presence signal returned
}


//--------------------------------------------------------------------------
// Funktion: unsigned char OneWireReadBit(void)
//
// Diese Funktion liest ein Bit vom Slave. 
//--------------------------------------------------------------------------
unsigned char OneWireReadBit(void)
{
    unsigned char i;
    unsigned char temp;
    // Master Start Flanke erzeugen
    DQ = LOW;             // pull DQ low to start timeslot
    DQ = HIGH;            // then return high
  
    for (i=0; i<3; i++); // delay 15us from start of timeslot

    // Status lesen
    temp = DQ;
    return(temp);          // return value of DQ line
}


//--------------------------------------------------------------------------
// Funktion: unsigned char OneWireReadByte(void)
//
// Diese Funktion liest ein Byte (LSB first) vom Slave. 
//--------------------------------------------------------------------------
unsigned char OneWireReadByte(void)
{
    unsigned char i;
    unsigned char byteValue = 0;
    unsigned char mask = 0x01;

    for (i=0;i<8;i++)
    {
        if(OneWireReadBit())  // Bit lesen 
            byteValue|=mask;
        mask <<= 1;
        
        OneWireDelay(6);       // Warten fuer den Rest des Timeslots
    }
    return(byteValue);
}


//--------------------------------------------------------------------------
// Funktion: unsigned char OneWireReadBit(void)
//
// Diese Funktion sendet ein Bit zum Slave. 
//--------------------------------------------------------------------------
void OneWireWriteBit(char bitValue)
{
    DQ = LOW;          // pull DQ low to start timeslot
    if(bitValue==1) 
        DQ =HIGH;      // return DQ high if write 1
    OneWireDelay(5);   // hold value for remainder of timeslot
    DQ = HIGH;
}


//--------------------------------------------------------------------------
// Funktion: void OneWireWriteByte(char byteVal)
//
// Diese Funktion sendet ein Byte (LSB first) zum Slave. 
//--------------------------------------------------------------------------
void OneWireWriteByte(char byteVal)
{
    unsigned char i;
    unsigned char temp;
    for (i=0; i<8; i++)         // writes byte, one bit at a time
    {
        temp = byteVal>>i;      // shifts val right 'i' spaces
        temp &= 0x01;           // copy that bit to temp
        OneWireWriteBit(temp);  // write bit in temp into
    }
    OneWireDelay(5);
}


//--------------------------------------------------------------------------
// Funktion: void delay(int time)
// Erzeugt Wartezeiten fuer das 1-Wire Bustiming
//--------------------------------------------------------------------------
void OneWireDelay(int time)
{
    int s;
    for (s=0; s < time; s++)
        ;
}
